%%% Algorithm 5.1, version optimized for time for large-scale butterflies  
% cf. algo5_1_opt_lambda.m for a more readable version, but suited for smaller
% matrices

function [l1opt]=algo5_1_opt_lambda_butterfly(x,y,t,dochopy)

%% Inputs
% x, y vectors to be quantized
% t number of bits 
% dochopy=1 if both x,y needs to be quantized
% dochopy=0 if just x needs to be quantized

%% Outputs
% l1opt optimal lambda, such that \hat{x}=chop(l1opt*x,t) and \hat{y}=chop(mu(\hat{x})*y,t)


if nargin<4
  dochopy=1;
end
d=length(x);



x12 = abs(x)./(2.^(floor(log2(abs(x)))));

% generate breakpoints
l1 = [1];
for i=1:d
  for e=1:2
    m=(2^(t-1)):(2^t-1);
    nextl = (m+0.5)*2^(e-t)/x12(i);
    nextl = nextl(nextl < 2 & nextl > 1);
    l1 = [l1 nextl];
   end
end

% sort breakpoints
l1 = sort(l1);


% find the optimum
L1=length(l1);
l1mid = (l1(1:end-1)+l1(2:end))/2;

xrep = repmat(x,1,L1-1);
xrep = xrep.*l1mid;
xrep = chop(xrep,t);

xtxrep = x'*xrep;
nrmxh2_rep = sum(xrep.^2);
l2rep = xtxrep./nrmxh2_rep;

if ~dochopy
  nrmy2 = norm(y)^2;
  ytyh_rep = nrmy2*l2rep;
  nrmyh2_rep = nrmy2*l2rep.^2;
  err_rep=nrmxh2_rep.*nrmyh2_rep - 2*xtxrep.*ytyh_rep;
else
  yhrep = repmat(y,1,L1-1);
  yhrep = l2rep.*yhrep;
  yhrep = chop(yhrep,t);
  ytyh_rep = y'*yhrep;
  nrmyh2_rep = sum(yhrep.^2);
  err_rep=nrmxh2_rep.*nrmyh2_rep - 2*xtxrep.*ytyh_rep;
end
[~,i] = min(err_rep);
l1opt = l1mid(i);
return






