% For various t and various n=2^p computes random butterfly factors 
% B_1,...,B_L nxn and quantizes
% them with t bits using Algorithm 7.2 (L-factor butterfly
% quantization: pairwise heuristic)
% Produces 'butterfly_nfactorsvaries_t%d_pairwise.mat'
%          'butterfly_%dfactors_tvaries_pairwise.mat'
% WARNING: may take long time to run

clear all; 
close all;
rng(1)

% Select number of factors p and number of bits t
pp = 4:2:16;
tt = 8:8;
start=tic;

err_rtn = zeros(length(pp),length(tt));
err_opt = zeros(length(pp),length(tt));
time_opt = zeros(length(pp),length(tt));

for ip=1:length(pp)
% size of the factors
p=pp(ip);
n=2^p;

% generate random butterfly factors
factors=cell(p,1);
for l=1:p
    n1 = 2^(l-1);
    A = speye(n1);
    B = [1 1 ; 1 1];
    n2 = n/(2^l);
    C = speye(n2);
    S = kron(kron(A,B),C);
    factors{l} = (sprand(S)-spones(S)*0.5);
end


% quantize
for it=1:length(tt)
t = tt(it);
fprintf('t=%d\n',t)

[q_factors,erropt,errrtn,ratio]=algo7_2_pairwise(factors,p,t);



err_opt(ip,it)=erropt;
err_rtn(ip,it)=errrtn;

end
end
total_time=toc(start)

%save(sprintf('butterfly_nfactorsvaries_t%d_pairwise.mat',t),'err_rtn','err_opt','time_opt');
%save(sprintf('butterfly_%dfactors_tvaries_pairwise.mat',p),'err_rtn','err_opt','time_opt');









